@@ -313,6 +313,14 @@ static int create_entity(cache_handle_t *h, cache_type_e type_e,
     cache_object_t *obj, *tmp_obj;
     mem_cache_object_t *mobj;
 
+    /* we don't support caching of range requests (yet) */
+    if (r->status == HTTP_PARTIAL_CONTENT) {
+        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+                     "disk_cache: URL %s partial content response not cached",
+                     key);
+        return DECLINED;
+    }
+
     if (len == -1) {
         /* Caching a streaming response. Assume the response is
          * less than or equal to max_streaming_buffer_size. We will
@@ -529,7 +537,7 @@ static int remove_url(cache_handle_t *h, apr_pool_t *p)
     }
 
     obj = h->cache_obj;
-    if (obj) {
+    if (obj && cache_find(sconf->cache_cache, obj->key) == obj) {
         cache_remove(sconf->cache_cache, obj);
         /* For performance, cleanup cache object after releasing the lock */
         cleanup = !apr_atomic_dec32(&obj->refcount);
@@ -669,12 +677,12 @@ static apr_status_t store_body(cache_handle_t *h, request_rec *r, apr_bucket_bri
     apr_read_type_e eblock = APR_BLOCK_READ;
     apr_bucket *e;
     char *cur;
-    int eos = 0;
 
     if (mobj->type == CACHE_TYPE_FILE) {
         apr_file_t *file = NULL;
         int fd = 0;
         int other = 0;
+        int eos = 0;
 
         /* We can cache an open file descriptor if:
          * - the brigade contains one and only one file_bucket &&
@@ -838,6 +846,15 @@ static apr_status_t store_body(cache_handle_t *h, request_rec *r, apr_bucket_bri
          */
         AP_DEBUG_ASSERT(obj->count <= mobj->m_len);
     }
+    if (r->connection->aborted && !obj->complete) {
+        ap_log_rerror(APLOG_MARK, APLOG_INFO, 0, r,
+                     "mem_cache: Discarding body for URL %s "
+                     "because client connection was aborted.",
+                     obj->key);
+        /* No need to cleanup - obj->complete unset, so
+         * decrement_refcount will discard the object */
+        return APR_EGENERAL;
+    }
     return APR_SUCCESS;
 }
 /**
