@@ -837,6 +837,7 @@ static int proxy_handler(request_rec *r)
     proxy_worker *worker = NULL;
     int attempts = 0, max_attempts = 0;
     struct dirconn_entry *list = (struct dirconn_entry *)conf->dirconn->elts;
+    int saved_status;
 
     /* is this for us? */
     if (!r->proxyreq || !r->filename || strncmp(r->filename, "proxy:", 6) != 0)
@@ -850,19 +851,15 @@ static int proxy_handler(request_rec *r)
             case M_TRACE: {
                 int access_status;
                 r->proxyreq = PROXYREQ_NONE;
-                if ((access_status = ap_send_http_trace(r)))
-                    ap_die(access_status, r);
-                else
-                    ap_finalize_request_protocol(r);
+                access_status = ap_send_http_trace(r);
+                ap_die(access_status, r);
                 return OK;
             }
             case M_OPTIONS: {
                 int access_status;
                 r->proxyreq = PROXYREQ_NONE;
-                if ((access_status = ap_send_http_options(r)))
-                    ap_die(access_status, r);
-                else
-                    ap_finalize_request_protocol(r);
+                access_status = ap_send_http_options(r);
+                ap_die(access_status, r);
                 return OK;
             }
             default: {
@@ -1084,10 +1081,30 @@ static int proxy_handler(request_rec *r)
     }
 cleanup:
     if (balancer) {
-        int post_status = proxy_run_post_request(worker, balancer, r, conf);
-        if (post_status == DECLINED) {
-            post_status = OK; /* no post_request handler available */
-            /* TODO: recycle direct worker */
+        /*
+         * Save current r->status and set it to the value of access_status which
+         * might be different (e.g. r->status could be HTTP_OK if e.g. we override
+         * the error page on the proxy or if the error was not generated by the
+         * backend itself but by the proxy e.g. a bad gateway) in order to give
+         * ap_proxy_post_request a chance to act correctly on the status code.
+         * But only do the above if access_status is not OK and not DONE, because
+         * in this case r->status might contain the true status and overwriting
+         * it with OK or DONE would be wrong.
+         */
+        if ((access_status != OK) && (access_status != DONE)) {
+            saved_status = r->status;
+            r->status = access_status;
+            proxy_run_post_request(worker, balancer, r, conf);
+            /*
+             * Only restore r->status if it has not been changed by
+             * ap_proxy_post_request as we assume that this change was intentional.
+             */
+            if (r->status == access_status) {
+                r->status = saved_status;
+            }
+        }
+        else {
+            ap_proxy_post_request(worker, balancer, r, conf);
         }
     }
 
@@ -1133,6 +1150,7 @@ static void * create_proxy_config(apr_pool_t *p, server_rec *s)
     ps->badopt = bad_error;
     ps->badopt_set = 0;
     ps->pool = p;
+    ps->s = s;
 
     return ps;
 }
@@ -1176,6 +1194,7 @@ static void * merge_proxy_config(apr_pool_t *p, void *basev, void *overridesv)
     ps->proxy_status = (overrides->proxy_status_set == 0) ? base->proxy_status : overrides->proxy_status;
     ps->proxy_status_set = overrides->proxy_status_set || base->proxy_status_set;
     ps->pool = p;
+    ps->s = overrides->s;
     return ps;
 }
 
